@@ -1,6 +1,6 @@
 /*
  * 
- * Copyright (C) 1999 Steve Ratcliffe
+ * Copyright (C) 1999-2003 Steve Ratcliffe
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -34,6 +34,7 @@
 #include <getopt.h>
 #include <string.h>
 #include <errno.h>
+#include <signal.h>
 
 /*
  * Play a midi file
@@ -51,12 +52,16 @@ static struct option long_opts[] = {
 	{"port", HAS_ARG, NULL, 'p'},
 	{"list", 0, NULL, 'l'},
 	{"delay", HAS_ARG, NULL, 'd'},
+	{"version", 0, NULL, 'v'},
 	{0, 0, 0, 0},
 };
 
 /* Delay at the end of a song */
 static int  delay = 2;
 
+/* Pointer to root context */
+seq_context_t *g_ctxp;
+
 /* Number of elements in an array */
 #define NELEM(a) ( sizeof(a)/sizeof((a)[0]) )
 
@@ -69,6 +74,9 @@ static void showlist();
 static void showusage();
 static void play(void *arg, struct element *el);
 static void no_errors_please(const char *file, int line, const char *function, int err, const char *fmt, ...);
+static void set_signal_handler(seq_context_t *ctxp);
+static void signal_handler(int sig);
+static void showversion(void);
 
 /*
  * Play a midi file
@@ -77,6 +85,7 @@ static void no_errors_please(const char *file, int line, const char *function, i
  *   -p, --port=client:port  - An ALSA client and port number to use
  *   -l, --list              - List possible output ports that could be used
  *   -d, --delay=SECS        - Delay after song ends (default 2 sec)
+ *   -v, --version           - Show version number
  *  Arguments:
  *    argc      - arg count
  *    argv      - arg vector
@@ -121,6 +130,9 @@ main(int argc, char **argv)
 			showlist();
 			exit(0);
 			break;
+		case 'v':
+			showversion();
+			exit(0);
 		case '?':
 			showusage();
 			exit(1);
@@ -145,11 +157,18 @@ main(int argc, char **argv)
 	if (ctxp == NULL)
 		return 1;
 
+	/* Set signal handler */
+	set_signal_handler(ctxp);
+
 	/* Now play all the files */
 	for (; optind < argc; optind++)
 		playfile(ctxp, argv[optind]);
 
 	seq_free_context(ctxp);
+
+	/* Restore signal handler */
+	signal(SIGINT, SIG_DFL);
+
 	return 0;
 }
 
@@ -359,6 +378,12 @@ N_("  -d delay        - Delay after song ends (default 2 sec)"),
 	}
 }
 
+static void
+showversion()
+{
+	printf("pmidi-%s\n", VERSION);
+}
+
 static void 
 play(void *arg, struct element *el)
 {
@@ -416,9 +441,34 @@ play(void *arg, struct element *el)
  * though such failed calls are part of normal operation.  So we have to
  * install our own handler to shut it up and prevent it messing up the list
  * output.
+ *
+ * Possibly not needed any more Dec 2003.
  */
 static void
 no_errors_please(const char *file, int line,
 		const char *function, int err, const char *fmt, ...)
 {
 }
+
+static void
+set_signal_handler(seq_context_t *ctxp)
+{
+	struct sigaction *sap = calloc(1, sizeof(struct sigaction));
+
+	g_ctxp = ctxp;
+
+	sap->sa_handler = signal_handler;
+	sigaction(SIGINT, sap, NULL);
+}
+
+/* signal handler */
+static void
+signal_handler(int sig)
+{
+	/* Close device */
+	if (g_ctxp) {
+		seq_free_context(g_ctxp);
+	}
+
+	exit(1);
+}
